/*
 * Copyright (C) 2012-2013 jeanfi@gmail.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#include <json/json.h>
#include "ttrss.h"
#include <webkit/webkit.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>

#include "phttp.h"
#include "webbrowser.h"

static const char *program_name;

static struct option long_options[] = {
	{"version", no_argument, 0, 'v'},
	{"help", no_argument, 0, 'h'},
	{0, 0, 0, 0}
};

static void print_version()
{
	printf("prss %s\n", VERSION);
	printf(_("Copyright (C) %s jeanfi@gmail.com\n"
		 "License GPLv2: GNU GPL version 2 or later "
		 "<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>\n"
		 "This is free software: you are free to change and "
		 " redistribute it.\n"
		 "There is NO WARRANTY, to the extent permitted by law.\n"),
	       "2012-2013");
}

static void print_help()
{
	printf(_("Usage: %s [OPTION]...\n"), program_name);

	puts(_("Prss is a GTK+ news reader for tt-rss."));

	puts("");
	puts(_("Options:"));
	puts(_("  -h, --help          display this help and exit\n"
	       "  -v, --version       display version information and exit"));

	puts("");

	printf(_("Report bugs to: %s\n"), PACKAGE_BUGREPORT);
	puts("");
	printf(_("%s home page: <%s>\n"), PACKAGE_NAME, PACKAGE_URL);
}

GtkTreeView *w_treeview;
GtkTreeView *w_headlineview;

void update()
{
	GSettings *settings;
	struct feed **feeds;
	GtkTreeModel *model;
	GtkTreeIter iter;

	printf("update\n");
	phttp_init();

	settings = g_settings_new("prss");

	ttrss_login(g_settings_get_string(settings, "url"),
		    g_settings_get_string(settings, "user"),
		    g_settings_get_string(settings, "password"));

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(w_treeview));
	gtk_list_store_clear(GTK_LIST_STORE(model));

	for (feeds = ttrss_get_feeds(); *feeds; feeds++) {
		gtk_list_store_append(GTK_LIST_STORE(model), &iter);

		gtk_list_store_set(GTK_LIST_STORE(model),
				   &iter,
				   0, (*feeds)->title,
				   1, (*feeds),
				   -1);

		printf("add\n");
	}
	printf("refresh done\n");
}

int feed_cursor_changed_cbk(GtkTreeView *treeview, gpointer data)
{
	GtkTreePath *path;
	GtkTreeViewColumn *cols;
	GtkTreeIter iter;
	GtkTreeModel *model, *headline_model;
	GtkListStore *headline_store;
	struct feed *feed;
	struct headline **headlines;
	char *title;

	printf("feed_cursor_changed_cbk\n");

	gtk_tree_view_get_cursor(treeview, &path, &cols);

	if (path) {
		model = gtk_tree_view_get_model(treeview);
		gtk_tree_model_get_iter(model, &iter, path);
		gtk_tree_model_get(model, &iter, 1, &feed, -1);

		headline_model = gtk_tree_view_get_model(w_headlineview);
		headline_store = GTK_LIST_STORE(headline_model);
		gtk_list_store_clear(headline_store);
		for (headlines = feed->headlines; *headlines; headlines++) {
			gtk_list_store_append(headline_store, &iter);

			if ((*headlines)->unread)
				title = g_strdup_printf("<b>%s</b>",
							(*headlines)->title);
			else
				title = strdup((*headlines)->title);

			gtk_list_store_set(headline_store,
					   &iter,
					   0, title,
					   1, (*headlines),
					   -1);

			free(title);
		}

		gtk_tree_path_free(path);
	}

	return 1;
}

int headline_cursor_changed_cbk(GtkTreeView *treeview, gpointer data)
{
	printf("headline_cursor_changed_cbk\n");

	GtkTreePath *path;
	GtkTreeViewColumn *cols;
	GtkTreeIter iter;
	GtkTreeModel *model;
	struct headline *headline;

	gtk_tree_view_get_cursor(treeview, &path, &cols);

	if (path) {
		model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
		gtk_tree_model_get_iter(model, &iter, path);
		gtk_tree_model_get(model, &iter, 1, &headline, -1);

		web_load(((struct headline *)headline)->content);

		gtk_tree_path_free(path);
	}

	return 1;
}

int main(int argc, char **argv)
{
	GtkWidget *window, *panh;
	GtkBuilder *builder;
	int optc, cmdok, opti;

	program_name = argv[0];

	setlocale(LC_ALL, "");

#if ENABLE_NLS
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);
#endif

	cmdok = 1;
	while ((optc = getopt_long(argc, argv, "vh", long_options,
				   &opti)) != -1) {
		switch (optc) {
		case 'h':
			print_help();
			exit(EXIT_SUCCESS);
		case 'v':
			print_version();
			exit(EXIT_SUCCESS);
		default:
			cmdok = 0;
			break;
		}
	}

	if (!cmdok || optind != argc) {
		fprintf(stderr, _("Try `%s --help' for more information.\n"),
			program_name);
		exit(EXIT_FAILURE);
	}

	gtk_init(NULL, NULL);
	builder = gtk_builder_new();
	gtk_builder_add_from_file
		(builder,
		 PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "prss.glade",
		 NULL);
	window = GTK_WIDGET(gtk_builder_get_object(builder, "window"));
	w_treeview = GTK_TREE_VIEW(gtk_builder_get_object(builder,
							  "feed_tree"));
	w_headlineview = GTK_TREE_VIEW(gtk_builder_get_object(builder,
							      "headline_tree"));

	gtk_builder_connect_signals(builder, NULL);

	update();

	panh = GTK_WIDGET(gtk_builder_get_object(builder, "webkit_window"));

	gtk_container_add(GTK_CONTAINER(panh), web_get_widget());

	g_object_unref(G_OBJECT(builder));

	gtk_widget_show_all(window);

	gtk_main();

	exit(EXIT_SUCCESS);
}
